'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var index = require('../codegen/baseline/index.js');
var index$1 = require('../runtime/index.js');
var parseExpressions = require('./utils/parse-expressions.js');

class Nimma {
  #fallback;
  #compiledFn;
  #module;
  #sourceCode;

  constructor(expressions, {
    fallback = null,
    unsafe = true,
    module = 'esm',
    npmProvider = null,
    customShorthands = null
  } = {}) {
    this.#fallback = fallback;
    this.#compiledFn = null;
    this.#module = module;
    this.#sourceCode = null;
    const {
      erroredExpressions,
      mappedExpressions
    } = parseExpressions['default'](expressions, unsafe, fallback !== null);
    this.tree = index['default'](mappedExpressions, {
      customShorthands,
      module,
      npmProvider
    });

    if (erroredExpressions.length > 0) {
      this.tree.attachFallbackExpressions(fallback, erroredExpressions);
    }
  }

  get sourceCode() {
    this.#sourceCode ?? (this.#sourceCode = String(this.tree.export(this.#module)));
    return this.#sourceCode;
  }

  query(input, callbacks) {
    this.#compiledFn ?? (this.#compiledFn = Function('module, require', `${String(this.tree.export('commonjs'))};return module.exports`)({}, id => {
      var _this$fallback;

      if (id === 'nimma/runtime') {
        return index$1;
      }

      return (_this$fallback = this.#fallback) === null || _this$fallback === void 0 ? void 0 : _this$fallback.runtimeDeps.get(id);
    }));
    this.#compiledFn(input, callbacks);
  }

}

exports['default'] = Nimma;
