import { variableDeclaration, variableDeclarator, callExpression, objectExpression, objectProperty, memberExpression, identifier, templateLiteral, templateElement, stringLiteral } from './ast/builders.mjs';

function getFunctionBody(fn) {
  const source = Reflect.apply(Function.toString, fn, []);
  const paramsDefEnd = source.indexOf(')') + 1;
  const body = source.slice(paramsDefEnd).replace(/^\s*(=>\s*)?/, '');
  const arr = source.slice(source.indexOf('('), paramsDefEnd).split(/[,\s]+/).splice(0, 3);
  return `${arr.join(', ')} => ${body}`;
}

class Fallback {
  #modules = new Map();
  #deps = new Map();
  #fn;
  #extraCode = '';
  runtimeDeps = new Map();

  constructor(deps, fn) {
    this.#fn = fn;

    for (const [source, specifiers] of Object.entries(deps)) {
      const members = new Map();
      const localModule = {};

      for (const {
        imported,
        local,
        value
      } of specifiers) {
        this.#deps.set(local, value);
        localModule[imported] = value;
        members.set(imported, local);
      }

      this.#modules.set(source, members);
      this.runtimeDeps.set(source, localModule);
    }
  }

  get extraCode() {
    this.#extraCode || (this.#extraCode = getFunctionBody(this.#fn));
    return this.#extraCode;
  }

  attach(tree) {
    for (const [source, members] of this.#modules.entries()) {
      tree.addModule(members, source);
    }

    const id = identifier('fallback');
    const args = Array.from(this.#deps.keys());
    tree.push(variableDeclaration('const', [variableDeclarator(id, callExpression(memberExpression(callExpression(identifier('Function'), [templateLiteral([templateElement({
      raw: `return ${this.extraCode}`
    })], [])]), identifier('call')), [objectExpression(args.map(arg => objectProperty(stringLiteral(arg), identifier(arg))))]))]), 'program');
    return id;
  }

}

export { Fallback as default };
