import baseline from '../codegen/baseline/index.mjs';
import * as index from '../runtime/index.mjs';
import parseExpressions from './utils/parse-expressions.mjs';

class Nimma {
  #fallback;
  #compiledFn;
  #module;
  #sourceCode;

  constructor(expressions, {
    fallback = null,
    unsafe = true,
    module = 'esm',
    npmProvider = null,
    customShorthands = null
  } = {}) {
    this.#fallback = fallback;
    this.#compiledFn = null;
    this.#module = module;
    this.#sourceCode = null;
    const {
      erroredExpressions,
      mappedExpressions
    } = parseExpressions(expressions, unsafe, fallback !== null);
    this.tree = baseline(mappedExpressions, {
      customShorthands,
      module,
      npmProvider
    });

    if (erroredExpressions.length > 0) {
      this.tree.attachFallbackExpressions(fallback, erroredExpressions);
    }
  }

  get sourceCode() {
    this.#sourceCode ?? (this.#sourceCode = String(this.tree.export(this.#module)));
    return this.#sourceCode;
  }

  query(input, callbacks) {
    this.#compiledFn ?? (this.#compiledFn = Function('module, require', `${String(this.tree.export('commonjs'))};return module.exports`)({}, id => {
      var _this$fallback;

      if (id === 'nimma/runtime') {
        return index;
      }

      return (_this$fallback = this.#fallback) === null || _this$fallback === void 0 ? void 0 : _this$fallback.runtimeDeps.get(id);
    }));
    this.#compiledFn(input, callbacks);
  }

}

export { Nimma as default };
